@@ -1,5 +1,5 @@
 from abc import ABC, abstractmethod
-from argparse import Namespace, ArgumentParser
+from argparse import ArgumentParser
 from copy import copy
 from functools import partial
 from queue import PriorityQueue
@@ -18,7 +18,7 @@ from rlai.rewards import Reward
 from rlai.runners.monitor import Monitor
 from rlai.states import State
 from rlai.states.mdp import MdpState
-from rlai.utils import IncrementalSampleAverager, sample_list_item
+from rlai.utils import IncrementalSampleAverager, sample_list_item, parse_arguments, get_base_argument_parser
 
 
 @rl_text(chapter=3, page=47)
@@ -63,7 +63,6 @@ class MdpEnvironment(Environment, ABC):
         :param agent: Agent.
         :return: 2-tuple of next state and next reward.
         """
-        pass
 
     @final
     def run_step(
@@ -255,314 +254,6 @@ class ModelBasedMdpEnvironment(MdpEnvironment, ABC):
         }
 
 
-@rl_text(chapter=3, page=60)
-class Gridworld(ModelBasedMdpEnvironment):
-    """
-    Gridworld MDP environment.
-    """
-
-    @staticmethod
-    def example_4_1(
-            random_state: RandomState
-    ):
-        """
-        Construct the Gridworld for Example 4.1.
-
-        :param random_state: Random state.
-        :return: Gridworld.
-        """
-
-        RR = [
-            Reward(
-                i=i,
-                r=r
-            )
-            for i, r in enumerate([0, -1])
-        ]
-
-        r_zero, r_minus_one = RR
-
-        g = Gridworld(
-            name='Example 4.1',
-            random_state=random_state,
-            T=None,
-            n_rows=4,
-            n_columns=4,
-            terminal_states=[(0, 0), (3, 3)],
-            RR=RR
-        )
-
-        # set nonterminal reward probabilities
-        for a in [g.a_up, g.a_down, g.a_left, g.a_right]:
-
-            # arrange grid such that a row-to-row scan will generate the appropriate state transition sequences for the
-            # current action.
-            if a == g.a_down:
-                grid = g.grid
-            elif a == g.a_up:
-                grid = np.flipud(g.grid)
-            elif a == g.a_right:
-                grid = g.grid.transpose()
-            elif a == g.a_left:
-                grid = np.flipud(g.grid.transpose())
-            else:
-                raise ValueError(f'Unknown action:  {a}')
-
-            # go row by row, with the final row transitioning to itself
-            for s_row_i, s_prime_row_i in zip(range(grid.shape[0]), list(range(1, grid.shape[0])) + [-1]):
-                for s, s_prime in zip(grid[s_row_i, :], grid[s_prime_row_i, :]):
-                    if not s.terminal:
-                        g.p_S_prime_R_given_S_A[s][a][s_prime][r_minus_one] = 1.0
-
-        # set terminal reward probabilities
-        for s in g.SS:
-            if s.terminal:
-                for a in s.AA:
-                    g.p_S_prime_R_given_S_A[s][a][s][r_zero] = 1.0
-
-        g.check_marginal_probabilities()
-
-        return g
-
-    @classmethod
-    def parse_arguments(
-            cls,
-            args
-    ) -> Tuple[Namespace, List[str]]:
-        """
-        Parse arguments.
-
-        :param args: Arguments.
-        :return: 2-tuple of parsed and unparsed arguments.
-        """
-
-        parsed_args, unparsed_args = super().parse_arguments(args)
-
-        parser = ArgumentParser(allow_abbrev=False)
-
-        parser.add_argument(
-            '--id',
-            type=str,
-            default='example_4_1',
-            help='Gridworld identifier.'
-        )
-
-        parsed_args, unparsed_args = parser.parse_known_args(unparsed_args, parsed_args)
-
-        return parsed_args, unparsed_args
-
-    @classmethod
-    def init_from_arguments(
-            cls,
-            args: List[str],
-            random_state: RandomState
-    ) -> Tuple[Environment, List[str]]:
-        """
-        Initialize an environment from arguments.
-
-        :param args: Arguments.
-        :param random_state: Random state.
-        :return: 2-tuple of an environment and a list of unparsed arguments.
-        """
-
-        parsed_args, unparsed_args = cls.parse_arguments(args)
-
-        gridworld = getattr(cls, parsed_args.id)(
-            random_state=random_state
-        )
-
-        return gridworld, unparsed_args
-
-    def __init__(
-            self,
-            name: str,
-            random_state: RandomState,
-            T: Optional[int],
-            n_rows: int,
-            n_columns: int,
-            terminal_states: List[Tuple[int, int]],
-            RR: List[Reward]
-    ):
-        """
-        Initialize the gridworld.
-
-        :param name: Name.
-        :param random_state: Random state.
-        :param T: Maximum number of steps to run, or None for no limit.
-        :param n_rows: Number of row.
-        :param n_columns: Number of columns.
-        :param terminal_states: List of terminal-state locations.
-        :param RR: List of all possible rewards.
-        """
-
-        AA = [
-            Action(
-                i=i,
-                name=direction
-            )
-            for i, direction in enumerate(['u', 'd', 'l', 'r'])
-        ]
-
-        self.a_up, self.a_down, self.a_left, self.a_right = AA
-
-        SS = [
-            MdpState(
-                i=row_i * n_columns + col_j,
-                AA=AA,
-                terminal=False
-            )
-            for row_i in range(n_rows)
-            for col_j in range(n_columns)
-        ]
-
-        for row, col in terminal_states:
-            SS[row * n_columns + col].terminal = True
-
-        super().__init__(
-            name=name,
-            random_state=random_state,
-            T=T,
-            SS=SS,
-            RR=RR
-        )
-
-        self.grid = np.array(self.SS).reshape(n_rows, n_columns)
-
-
-@rl_text(chapter=4, page=84)
-class GamblersProblem(ModelBasedMdpEnvironment):
-    """
-    Gambler's problem MDP environment.
-    """
-
-    @classmethod
-    def parse_arguments(
-            cls,
-            args
-    ) -> Tuple[Namespace, List[str]]:
-        """
-        Parse arguments.
-
-        :param args: Arguments.
-        :return: 2-tuple of parsed and unparsed arguments.
-        """
-
-        parsed_args, unparsed_args = super().parse_arguments(args)
-
-        parser = ArgumentParser(allow_abbrev=False)
-
-        parser.add_argument(
-            '--p-h',
-            type=float,
-            default=0.5,
-            help='Probability of coin toss coming up heads.'
-        )
-
-        parsed_args, unparsed_args = parser.parse_known_args(unparsed_args, parsed_args)
-
-        return parsed_args, unparsed_args
-
-    @classmethod
-    def init_from_arguments(
-            cls,
-            args: List[str],
-            random_state: RandomState
-    ) -> Tuple[Environment, List[str]]:
-        """
-        Initialize an environment from arguments.
-
-        :param args: Arguments.
-        :param random_state: Random state.
-        :return: 2-tuple of an environment and a list of unparsed arguments.
-        """
-
-        parsed_args, unparsed_args = cls.parse_arguments(args)
-
-        gamblers_problem = GamblersProblem(
-            name=f"gambler's problem (p={parsed_args.p_h})",
-            random_state=random_state,
-            **vars(parsed_args)
-        )
-
-        return gamblers_problem, unparsed_args
-
-    def __init__(
-            self,
-            name: str,
-            random_state: RandomState,
-            T: Optional[int],
-            p_h: float
-    ):
-        """
-        Initialize the MDP environment.
-
-        :param name: Name.
-        :param random_state: Random state.
-        :param T: Maximum number of steps to run, or None for no limit.
-        :param p_h: Probability of coin toss coming up heads.
-        """
-
-        self.p_h = p_h
-        self.p_t = 1 - p_h
-
-        # the range of possible actions:  stake 0 (no play) through 50 (at capital=50). beyond a capital of 50 the
-        # agent is only allowed to stake an amount that would take them to 100 on a win.
-        AA = [Action(i=stake, name=f'Stake {stake}') for stake in range(0, 51)]
-
-        # two possible rewards:  0.0 and 1.0
-        self.r_not_win = Reward(0, 0.0)
-        self.r_win = Reward(1, 1.0)
-        RR = [self.r_not_win, self.r_win]
-
-        # range of possible states (capital levels)
-        SS = [
-            MdpState(
-                i=capital,
-
-                # the range of permissible actions is state dependent
-                AA=[
-                    a
-                    for a in AA
-                    if a.i <= min(capital, 100 - capital)
-                ],
-
-                terminal=capital == 0 or capital == 100
-            )
-
-            # include terminal capital levels of 0 and 100
-            for capital in range(0, 101)
-        ]
-
-        super().__init__(
-            name=name,
-            random_state=random_state,
-            T=T,
-            SS=SS,
-            RR=RR
-        )
-
-        for s in self.SS:
-            for a in self.p_S_prime_R_given_S_A[s]:
-
-                # next state and reward if heads
-                s_prime_h = self.SS[s.i + a.i]
-                if s_prime_h.i > 100:
-                    raise ValueError('Expected state to be <= 100')
-
-                r_h = self.r_win if not s.terminal and s_prime_h.i == 100 else self.r_not_win
-                self.p_S_prime_R_given_S_A[s][a][s_prime_h][r_h] = self.p_h
-
-                # next state and reward if tails
-                s_prime_t = self.SS[s.i - a.i]
-                if s_prime_t.i < 0:
-                    raise ValueError('Expected state to be >= 0')
-
-                r_t = self.r_win if not s.terminal and s_prime_t.i == 100 else self.r_not_win
-                self.p_S_prime_R_given_S_A[s][a][s_prime_t][r_t] += self.p_t  # add the probability, in case the results of head and tail are the same.
-
-        self.check_marginal_probabilities()
-
-
 @rl_text(chapter=8, page=159)
 class MdpPlanningEnvironment(MdpEnvironment, ABC):
     """
@@ -571,21 +262,19 @@ class MdpPlanningEnvironment(MdpEnvironment, ABC):
     """
 
     @classmethod
-    def parse_arguments(
-            cls,
-            args
-    ) -> Tuple[Namespace, List[str]]:
+    def get_argument_parser(
+            cls
+    ) -> ArgumentParser:
         """
-        Parse arguments.
+        Get argument parser.
 
-        :param args: Arguments.
-        :return: 2-tuple of parsed and unparsed arguments.
+        :return: Argument parser.
         """
 
-        # don't call super's argument parser, so that we do not pick up the --T argument intended for the actual
+        # don't use super's argument parser, so that we do not pick up the --T argument intended for the actual
         # environment. we're going to use --T-planning instead (see below).
 
-        parser = ArgumentParser(allow_abbrev=False)
+        parser = get_base_argument_parser()
 
         parser.add_argument(
             '--T-planning',
@@ -599,9 +288,7 @@ class MdpPlanningEnvironment(MdpEnvironment, ABC):
             help='Number of planning improvements to make for each direct improvement.'
         )
 
-        parsed_args, unparsed_args = parser.parse_known_args(args)
-
-        return parsed_args, unparsed_args
+        return parser
 
     def reset_for_new_run(
             self,
@@ -655,30 +342,29 @@ class PrioritizedSweepingMdpPlanningEnvironment(MdpPlanningEnvironment):
     """
 
     @classmethod
-    def parse_arguments(
-            cls,
-            args
-    ) -> Tuple[Namespace, List[str]]:
+    def get_argument_parser(
+            cls
+    ) -> ArgumentParser:
         """
-        Parse arguments.
+        Get argument parser.
 
-        :param args: Arguments.
-        :return: 2-tuple of parsed and unparsed arguments.
+        :return: Argument parser.
         """
 
-        parsed_args, unparsed_args = super().parse_arguments(args)
-
-        parser = ArgumentParser(allow_abbrev=False)
+        parser = ArgumentParser(
+            prog=f'{cls.__module__}.{cls.__name__}',
+            parents=[super().get_argument_parser()],
+            allow_abbrev=False,
+            add_help=False
+        )
 
         parser.add_argument(
             '--priority-theta',
             type=float,
-            help='Threshold on priority values, below which state-action pairs are added to the priority queue.'
+            help='Threshold on priority values, below which state-action pairs are added to the priority queue. Note that all priority values are negative, with higher priorities being larger negative values.'
         )
 
-        parsed_args, unparsed_args = parser.parse_known_args(unparsed_args, parsed_args)
-
-        return parsed_args, unparsed_args
+        return parser
 
     @classmethod
     def init_from_arguments(
@@ -694,7 +380,7 @@ class PrioritizedSweepingMdpPlanningEnvironment(MdpPlanningEnvironment):
         :return: 2-tuple of an environment and a list of unparsed arguments.
         """
 
-        parsed_args, unparsed_args = cls.parse_arguments(args)
+        parsed_args, unparsed_args = parse_arguments(cls, args)
 
         planning_environment = PrioritizedSweepingMdpPlanningEnvironment(
             name=f"prioritized planning (theta={parsed_args.priority_theta})",
@@ -722,18 +408,19 @@ class PrioritizedSweepingMdpPlanningEnvironment(MdpPlanningEnvironment):
         :return: 2-tuple of (1) 3-tuple of current state, current action, and next-state, and (2) reward.
         """
 
-        planning_state, a = self.get_state_action_with_highest_priority()
+        planning_state, planning_a = self.get_state_action_with_highest_priority()
 
         # if there is nothing left in the priority queue, then the planning episode is done.
         if planning_state is None:
             planning_state = state
+            planning_a = a
             next_state = copy(state)
             next_state.terminal = True
             r = 0.0
         else:
 
             # sample next state and reward from model
-            next_state, r = self.model.sample_next_state_and_reward(planning_state, a, self.random_state)
+            next_state, r = self.model.sample_next_state_and_reward(planning_state, planning_a, self.random_state)
 
             # add predecessors into priority queue
             for pred_state, pred_action, r in self.get_predecessor_state_action_rewards(planning_state):
@@ -741,7 +428,7 @@ class PrioritizedSweepingMdpPlanningEnvironment(MdpPlanningEnvironment):
                 priority = -abs(target_value - self.q_S_A[pred_state][pred_action].get_value())
                 self.add_state_action_priority(pred_state, pred_action, priority)
 
-        return (planning_state, a, next_state), Reward(None, r)
+        return (planning_state, planning_a, next_state), Reward(None, r)
 
     def get_predecessor_state_action_rewards(
             self,
@@ -847,6 +534,25 @@ class TrajectorySamplingMdpPlanningEnvironment(MdpPlanningEnvironment):
     explored during planning.
     """
 
+    @classmethod
+    def get_argument_parser(
+            cls
+    ) -> ArgumentParser:
+        """
+        Get argument parser.
+
+        :return: Argument parser.
+        """
+
+        parser = ArgumentParser(
+            prog=f'{cls.__module__}.{cls.__name__}',
+            parents=[super().get_argument_parser()],
+            allow_abbrev=False,
+            add_help=False
+        )
+
+        return parser
+
     @classmethod
     def init_from_arguments(
             cls,
@@ -861,7 +567,7 @@ class TrajectorySamplingMdpPlanningEnvironment(MdpPlanningEnvironment):
         :return: 2-tuple of an environment and a list of unparsed arguments.
         """
 
-        parsed_args, unparsed_args = cls.parse_arguments(args)
+        parsed_args, unparsed_args = parse_arguments(cls, args)
 
         planning_environment = TrajectorySamplingMdpPlanningEnvironment(
             name=f"trajectory planning",
