@@ -5,7 +5,7 @@ from datetime import datetime
 from typing import Optional
 
 from rlai.agents.mdp import MdpAgent
-from rlai.environments.mdp import MdpEnvironment
+from rlai.environments.mdp import MdpEnvironment, MdpPlanningEnvironment
 from rlai.environments.openai_gym import Gym
 from rlai.gpi.monte_carlo.evaluation import evaluate_q_pi
 from rlai.gpi.utils import plot_policy_iteration
@@ -21,6 +21,7 @@ def iterate_value_q_pi(
         num_episodes_per_improvement: int,
         update_upon_every_visit: bool,
         epsilon: Optional[float],
+        planning_environment: Optional[MdpPlanningEnvironment],
         make_final_policy_greedy: bool,
         q_S_A: StateActionValueEstimator,
         off_policy_agent: Optional[MdpAgent] = None,
@@ -40,6 +41,7 @@ def iterate_value_q_pi(
     :param update_upon_every_visit: See `rlai.gpi.monte_carlo.evaluation.evaluate_q_pi`.
     :param epsilon: Total probability mass to spread across all actions, resulting in an epsilon-greedy policy. Must
     be >= 0 if provided.
+    :param planning_environment: Not support. Will raise exception if passed.
     :param make_final_policy_greedy: Whether or not to make the agent's final policy greedy with respect to the q-values
     that have been learned, regardless of the value of epsilon used to estimate the q-values.
     :param q_S_A: State-action value estimator.
@@ -51,6 +53,9 @@ def iterate_value_q_pi(
     :param checkpoint_path: Checkpoint path. Must be provided if `num_improvements_per_checkpoint` is provided.
     """
 
+    if planning_environment is not None:
+        raise ValueError('Planning environments are not currently supported for Monte Carlo iteration.')
+
     if (epsilon is None or epsilon == 0.0) and off_policy_agent is None:
         warnings.warn('epsilon is 0.0 and there is no off-policy agent. Exploration and convergence not guaranteed. Consider passing epsilon > 0 or a soft off-policy agent to maintain exploration.')
 
@@ -60,7 +65,7 @@ def iterate_value_q_pi(
     i = 0
     iteration_average_reward = []
     iteration_total_states = []
-    iteration_num_states_updated = []
+    iteration_num_states_improved = []
     elapsed_seconds_average_rewards = {}
     start_datetime = datetime.now()
     while i < num_improvements:
@@ -77,7 +82,7 @@ def iterate_value_q_pi(
             off_policy_agent=off_policy_agent
         )
 
-        num_states_updated = q_S_A.update_policy(
+        num_states_improved = q_S_A.improve_policy(
             agent=agent,
             states=evaluated_states,
             epsilon=epsilon
@@ -85,7 +90,7 @@ def iterate_value_q_pi(
 
         iteration_average_reward.append(average_reward)
         iteration_total_states.append(len(q_S_A))
-        iteration_num_states_updated.append(num_states_updated)
+        iteration_num_states_improved.append(num_states_improved)
 
         elapsed_seconds = int((datetime.now() - start_datetime).total_seconds())
         if elapsed_seconds not in elapsed_seconds_average_rewards:
@@ -96,7 +101,7 @@ def iterate_value_q_pi(
         i += 1
 
         if num_improvements_per_plot is not None and i % num_improvements_per_plot == 0:
-            plot_policy_iteration(iteration_average_reward, iteration_total_states, iteration_num_states_updated, elapsed_seconds_average_rewards)
+            plot_policy_iteration(iteration_average_reward, iteration_total_states, iteration_num_states_improved, elapsed_seconds_average_rewards)
 
         if num_improvements_per_checkpoint is not None and i % num_improvements_per_checkpoint == 0:
 
@@ -113,11 +118,13 @@ def iterate_value_q_pi(
                 'num_episodes_per_improvement': num_episodes_per_improvement,
                 'update_upon_every_visit': update_upon_every_visit,
                 'epsilon': epsilon,
+                'planning_environment': planning_environment,
                 'make_final_policy_greedy': make_final_policy_greedy,
                 'q_S_A': q_S_A,
                 'off_policy_agent': off_policy_agent,
                 'num_improvements_per_plot': num_improvements_per_plot,
-                'num_improvements_per_checkpoint': num_improvements_per_checkpoint
+                'num_improvements_per_checkpoint': num_improvements_per_checkpoint,
+                'checkpoint_path': checkpoint_path
             }
 
             with open(checkpoint_path, 'wb') as checkpoint_file:
@@ -129,7 +136,7 @@ def iterate_value_q_pi(
     print(f'Value iteration of q_pi terminated after {i} iteration(s).')
 
     if make_final_policy_greedy:
-        q_S_A.update_policy(
+        q_S_A.improve_policy(
             agent=agent,
             states=None,
             epsilon=0.0
