@@ -18,6 +18,7 @@ def iterate_value_q_pi(
         environment: MdpEnvironment,
         num_improvements: int,
         num_episodes_per_improvement: int,
+        num_updates_per_improvement: Optional[int],
         alpha: Optional[float],
         mode: Union[Mode, str],
         n_steps: Optional[int],
@@ -35,8 +36,10 @@ def iterate_value_q_pi(
     :param agent: Agent.
     :param environment: Environment.
     :param num_improvements: Number of policy improvements to make.
-    :param num_episodes_per_improvement: Number of policy evaluation episodes to execute for each iteration of
+    :param num_episodes_per_improvement: Number of policy evaluation episodes to execute for each iteration of policy
     improvement.
+    :param num_updates_per_improvement: Number of state-action value updates to execute for each iteration of policy
+    improvement, or None for policy improvement per specified number of episodes.
     :param alpha: Constant step size to use when updating Q-values, or None for 1/n step size.
     :param mode: Evaluation mode (see `rlai.gpi.temporal_difference.evaluation.Mode`).
     :param n_steps: Number of steps (see `rlai.gpi.temporal_difference.evaluation.evaluate_q_pi`).
@@ -64,7 +67,7 @@ def iterate_value_q_pi(
     i = 0
     iteration_average_reward = []
     iteration_total_states = []
-    iteration_num_states_updated = []
+    iteration_num_states_improved = []
     elapsed_seconds_average_rewards = {}
     start_datetime = datetime.now()
     while i < num_improvements:
@@ -76,6 +79,8 @@ def iterate_value_q_pi(
             agent=agent,
             environment=environment,
             num_episodes=num_episodes_per_improvement,
+            num_updates_per_improvement=num_updates_per_improvement,
+            epsilon=epsilon,
             alpha=alpha,
             mode=mode,
             n_steps=n_steps,
@@ -83,15 +88,17 @@ def iterate_value_q_pi(
             q_S_A=q_S_A
         )
 
-        num_states_updated = q_S_A.update_policy(
+        num_states_improved = q_S_A.improve_policy(
             agent=agent,
             states=evaluated_states,
             epsilon=epsilon
         )
 
+        q_S_A.plot(final=False)
+
         iteration_average_reward.append(average_reward)
         iteration_total_states.append(len(q_S_A))
-        iteration_num_states_updated.append(num_states_updated)
+        iteration_num_states_improved.append(num_states_improved)
 
         # run planning through a recursive call to the iteration method, passing the planning environment as the
         # environment to interact with and disabling planning in the recursive call.
@@ -102,6 +109,7 @@ def iterate_value_q_pi(
                 environment=planning_environment,
                 num_improvements=planning_environment.num_planning_improvements_per_direct_improvement,
                 num_episodes_per_improvement=num_episodes_per_improvement,
+                num_updates_per_improvement=num_updates_per_improvement,
                 alpha=alpha,
                 mode=mode,
                 n_steps=n_steps,
@@ -124,7 +132,7 @@ def iterate_value_q_pi(
         i += 1
 
         if num_improvements_per_plot is not None and i % num_improvements_per_plot == 0:
-            plot_policy_iteration(iteration_average_reward, iteration_total_states, iteration_num_states_updated, elapsed_seconds_average_rewards)
+            plot_policy_iteration(iteration_average_reward, iteration_total_states, iteration_num_states_improved, elapsed_seconds_average_rewards)
 
         if num_improvements_per_checkpoint is not None and i % num_improvements_per_checkpoint == 0:
 
@@ -143,6 +151,7 @@ def iterate_value_q_pi(
                 'environment': environment,
                 'num_improvements': num_improvements - i,
                 'num_episodes_per_improvement': num_episodes_per_improvement,
+                'num_updates_per_improvement': num_updates_per_improvement,
                 'alpha': alpha,
                 'mode': mode,
                 'n_steps': n_steps,
@@ -151,7 +160,8 @@ def iterate_value_q_pi(
                 'make_final_policy_greedy': make_final_policy_greedy,
                 'q_S_A': q_S_A,
                 'num_improvements_per_plot': num_improvements_per_plot,
-                'num_improvements_per_checkpoint': num_improvements_per_checkpoint
+                'num_improvements_per_checkpoint': num_improvements_per_checkpoint,
+                'checkpoint_path': checkpoint_path
             }
 
             with open(checkpoint_path, 'wb') as checkpoint_file:
@@ -162,8 +172,10 @@ def iterate_value_q_pi(
 
     print(f'Value iteration of q_pi terminated after {i} iteration(s).')
 
+    q_S_A.plot(final=True)
+
     if make_final_policy_greedy:
-        q_S_A.update_policy(
+        q_S_A.improve_policy(
             agent=agent,
             states=None,
             epsilon=0.0
